"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoopyLoop = void 0;
const events_1 = require("events");
const _setImmediate = globalThis.setImmediate
    || ((fn) => setTimeout(fn, 0));
class LoopyLoop extends events_1.EventEmitter {
    _stop;
    _running;
    _runTask;
    constructor(task, opts = { maxChained: 10 }) {
        super();
        this._stop = false;
        this._running = false;
        this._runTask = typeof opts?.maxChained === 'number' && opts.maxChained > 1
            ? () => new Promise((resolve, reject) => this._chain(task, opts.maxChained, resolve, reject))
            : () => task();
    }
    isRunning() {
        return this._running;
    }
    start() {
        if (!this._running) {
            this._stop = false;
            this._running = true;
            this.emit('started');
            _setImmediate(() => {
                this._loop();
            });
        }
        return this;
    }
    stop() {
        if (this._running) {
            this._stop = true;
        }
        return this;
    }
    _loop() {
        this._runTask()
            .then(() => {
            if (this._stop) {
                this._running = false;
                this._stop = false;
                this.emit('stopped');
                return;
            }
            _setImmediate(() => this._loop());
        })
            .catch((err) => {
            this._running = false;
            this._stop = false;
            this.emit('error', err);
            this.emit('stopped');
        });
    }
    _chain = (task, count, resolve, reject) => {
        if (count < 1 || this._stop) {
            resolve();
            return;
        }
        task()
            .then(() => this._chain(task, count - 1, resolve, reject))
            .catch(reject);
    };
}
exports.LoopyLoop = LoopyLoop;
//# sourceMappingURL=LoopyLoop.js.map