"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_test_1 = require("node:test");
const node_assert_1 = __importStar(require("node:assert"));
const LoopyLoop_js_1 = require("./LoopyLoop.js");
const wait = (delay) => {
    return new Promise((resolve) => {
        setTimeout(resolve, delay);
    });
};
(0, node_test_1.test)('constructor', (t) => {
    const task = (() => Promise.resolve());
    const loop = new LoopyLoop_js_1.LoopyLoop(task);
    (0, node_assert_1.default)(loop instanceof LoopyLoop_js_1.LoopyLoop);
});
(0, node_test_1.test)('start(), stop(), isRunning()', (t, done) => {
    let work = false;
    const task = (() => {
        work = true;
        return Promise.resolve();
    });
    const loop = new LoopyLoop_js_1.LoopyLoop(task);
    loop.on('started', () => {
        (0, node_assert_1.strictEqual)(loop.isRunning(), true);
        process.nextTick(() => {
            loop.stop();
        });
    });
    loop.on('stopped', () => {
        (0, node_assert_1.strictEqual)(loop.isRunning(), false);
        done();
    });
    loop.start();
});
(0, node_test_1.test)('should not start more than once', (t, done) => {
    const task = () => Promise.resolve();
    let count = 0;
    const loop = new LoopyLoop_js_1.LoopyLoop(task)
        .on('started', () => {
        count += 1;
    })
        .on('stopped', () => {
        (0, node_assert_1.strictEqual)(count, 1);
        done();
    });
    setTimeout(() => loop.stop(), 10);
    loop.start();
    loop.start();
});
(0, node_test_1.test)('should not stop more than once', (t, done) => {
    const task = () => Promise.resolve();
    let count = 0;
    const loop = new LoopyLoop_js_1.LoopyLoop(task)
        .on('stopped', () => {
        count += 1;
    });
    setTimeout(() => {
        (0, node_assert_1.strictEqual)(count, 1);
        done();
    }, 20);
    setTimeout(() => { loop.stop(); }, 5);
    setTimeout(() => { loop.stop(); }, 10);
    loop.start();
});
(0, node_test_1.test)('sequence - w/o chained calls', (t, done) => {
    let seq = 0;
    const target = [];
    const source = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];
    const loop = new LoopyLoop_js_1.LoopyLoop(() => {
        const localSeq = seq++;
        return wait(Math.random() * 50).then(() => {
            target.push(localSeq);
            if (target.length >= 10) {
                loop.stop();
            }
        });
    }, { maxChained: 0 });
    loop.on('stopped', () => {
        (0, node_assert_1.deepStrictEqual)(source, target);
        done();
    });
    loop.start();
});
(0, node_test_1.test)('sequence - w/ chained calls', (t, done) => {
    let seq = 0;
    const target = [];
    const source = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];
    const loop = new LoopyLoop_js_1.LoopyLoop(() => {
        const localSeq = seq++;
        return wait(Math.random() * 50).then(() => {
            target.push(localSeq);
            if (target.length >= 10) {
                loop.stop();
            }
        });
    }, { maxChained: 2 });
    loop.on('stopped', () => {
        (0, node_assert_1.deepStrictEqual)(source, target);
        done();
    });
    loop.start();
});
(0, node_test_1.test)('catch error w/o chained calls', (t, done) => {
    let i = 0;
    const err = new Error('test error');
    const task = (() => {
        if (++i === 10) {
            return Promise.reject(err);
        }
        return Promise.resolve();
    });
    const loop = new LoopyLoop_js_1.LoopyLoop(task, { maxChained: 0 });
    loop.on('error', (_err) => {
        (0, node_assert_1.strictEqual)(err, _err);
        done();
    });
    loop.start();
});
(0, node_test_1.test)('catch error w/ chained calls', (t, done) => {
    let i = 0;
    const err = new Error('test error');
    const task = (() => {
        if (++i === 10) {
            return Promise.reject(err);
        }
        return Promise.resolve();
    });
    const loop = new LoopyLoop_js_1.LoopyLoop(task, { maxChained: Infinity });
    loop.on('error', (_err) => {
        (0, node_assert_1.strictEqual)(err, _err);
        done();
    });
    loop.start();
});
//# sourceMappingURL=LoopyLoop.spec.js.map